@@ -230,6 +230,8 @@ def train(params, train_set, num_boost_round=100,
 
     if gp_model is not None:
         # some checks
+        if gp_model.has_covariates:
+            raise ValueError("The 'gp_model' cannot have covariates 'X' (a linear predictor) in the GPBoost algorithm.")
         if use_gp_model_for_validation and not not valid_sets and not not feval:
             raise ValueError("use_gp_model_for_validation=True is currently "
                              "not supported for custom validation functions. If you need this feature, contact the "
@@ -456,28 +458,22 @@ def _make_n_folds(full_data, folds, nfold, params, seed, gp_model=None, use_gp_m
             if gp_model.cluster_ids is not None:
                 cluster_ids = gp_model.cluster_ids[train_idx]
                 cluster_ids_pred = gp_model.cluster_ids[test_idx]
-            vecchia_approx = gp_model.vecchia_approx
-            num_neighbors = gp_model.num_neighbors
-            vecchia_ordering = gp_model.vecchia_ordering
-            vecchia_pred_type = gp_model.vecchia_pred_type
-            num_neighbors_pred = gp_model.num_neighbors_pred
-            cov_function = gp_model.cov_function
-            cov_fct_shape = gp_model.cov_fct_shape
-            cov_fct_taper_range = gp_model.cov_fct_taper_range
-            ind_effect_group_rand_coef = gp_model.ind_effect_group_rand_coef
-            drop_intercept_group_rand_effect = gp_model.drop_intercept_group_rand_effect
             gp_model_train = GPModel(group_data=group_data,
                                      group_rand_coef_data=group_rand_coef_data,
-                                     ind_effect_group_rand_coef=ind_effect_group_rand_coef,
-                                     drop_intercept_group_rand_effect=drop_intercept_group_rand_effect,
+                                     ind_effect_group_rand_coef=gp_model.ind_effect_group_rand_coef,
+                                     drop_intercept_group_rand_effect=gp_model.drop_intercept_group_rand_effect,
                                      gp_coords=gp_coords,
                                      gp_rand_coef_data=gp_rand_coef_data,
-                                     cov_function=cov_function,
-                                     cov_fct_shape=cov_fct_shape,
-                                     cov_fct_taper_range=cov_fct_taper_range,
-                                     vecchia_approx=vecchia_approx,
-                                     num_neighbors=num_neighbors,
-                                     vecchia_ordering=vecchia_ordering,
+                                     cov_function=gp_model.cov_function,
+                                     cov_fct_shape=gp_model.cov_fct_shape,
+                                     gp_approx=gp_model.gp_approx,
+                                     cov_fct_taper_range=gp_model.cov_fct_taper_range,
+                                     cov_fct_taper_shape=gp_model.cov_fct_taper_shape,
+                                     num_neighbors=gp_model.num_neighbors,
+                                     vecchia_ordering=gp_model.vecchia_ordering,
+                                     num_ind_points=gp_model.num_ind_points,
+                                     matrix_inversion_method=gp_model.matrix_inversion_method,
+                                     seed=gp_model.seed,
                                      cluster_ids=cluster_ids,
                                      likelihood=gp_model._get_likelihood_name(),
                                      free_raw_data=True)
@@ -487,8 +483,8 @@ def _make_n_folds(full_data, folds, nfold, params, seed, gp_model=None, use_gp_m
                                                    gp_coords_pred=gp_coords_pred,
                                                    gp_rand_coef_data_pred=gp_rand_coef_data_pred,
                                                    cluster_ids_pred=cluster_ids_pred,
-                                                   vecchia_pred_type=vecchia_pred_type,
-                                                   num_neighbors_pred=num_neighbors_pred,
+                                                   vecchia_pred_type=gp_model.vecchia_pred_type,
+                                                   num_neighbors_pred=gp_model.num_neighbors_pred,
                                                    cg_delta_conv_pred=gp_model.cg_delta_conv_pred)
             cvbooster = Booster(params=tparam, train_set=train_set, gp_model=gp_model_train)
             gp_model._set_likelihood(
